@@ -12,6 +12,8 @@ CANONICAL_MODEL_NAME = 'CanonicalName'
 ESTIMATOR_TYPE = 'EstimatorType'
 MODEL_COMMIT = 'ModelCommit'
 TRAINING_JOB_NAME = 'TrainingJobName'
+CLUSTER_NAME = 'ClusterName'
+MODE = 'Mode'
 GIT_REPO_URL = 'GitRepoUrl'
 GIT_REPO_NAME = 'GitRepoName'
 CODEBUILD_URL = 'CodeBuildUrl'
